# -*- encoding: utf-8 -*-

"""Unittests for our helpers module."""

from __future__ import absolute_import, unicode_literals

import datetime as dt

import dbus
import pytest
from hamster_lib import Activity, Category, Fact, Tag

from hamster_dbus import helpers
from hamster_dbus.helpers import DBusActivity, DBusCategory, DBusFact, DBusTag


@pytest.mark.parametrize(('value, expectation'), ((None, -1), (1, 1), ('1', 1)))
def test__none_to_int(value, expectation):
    """
    Make sure ``None`` is converted to an integer representation.

    All other values should be returned as unmodified integers.
    """
    result = helpers._none_to_int(value)
    assert result == expectation


@pytest.mark.parametrize(('value, expectation'), ((-1, None), (1, 1), ('1', 1)))
def test__int_to_none(value, expectation):
    """
    Make sure integer designating ``None`` gets converted properly.

    All other values should be returned as unmodified integers.
    """
    result = helpers._int_to_none(value)
    assert result == expectation


@pytest.mark.parametrize(('value', 'expectation'), (
    (dt.datetime(2017, 1, 1, 18, 50, 3), '2017-01-01 18:50:03'),
    (dt.date(2017, 12, 1), '2017-12-01'),
    (dt.time(18, 2, 1), '18:02:01'),
    (None, ''),
))
def test_datetime_to_text(value, expectation):
    """Make sure ``datetime`` instances are converted as expected."""
    result = helpers.datetime_to_text(value)
    assert result == expectation


@pytest.mark.parametrize(('value', 'expectation'), (
    ('2017-01-01 18:50:03', dt.datetime(2017, 1, 1, 18, 50, 3)),
    ('2017-12-01', dt.date(2017, 12, 1)),
    ('18:02:01', dt.time(18, 2, 1)),
    ('', None),
))
def test_text_to_datetime(value, expectation):
    """Make sure text representing``datetime`` instances is converted as expected."""
    result = helpers.text_to_datetime(value)
    assert result == expectation


def test_datetime_to_text_non_datetime():
    """Make sure an error is thrown if we pass invalid instances."""
    with pytest.raises(TypeError):
        helpers.datetime_to_text('foobar')


@pytest.mark.parametrize(('category', 'expectation'), (
    (Category('foo', 1), helpers.DBusCategory(1, 'foo')),
    (Category('foo', None), helpers.DBusCategory(-1, 'foo')),
    (None, helpers.DBusCategory(-2, '')),
))
def test_hamster_to_dbus_category(category, expectation):
    """Make sure that serialization works as intended."""
    result = helpers.hamster_to_dbus_category(category)
    assert result == expectation
    assert isinstance(result, DBusCategory)


@pytest.mark.parametrize(('category_tuple', 'expectation'), (
    ((1, 'foo'), Category('foo', 1)),
    ((-1, 'foo'), Category('foo', None)),
    ((-2, ''), None),
    (helpers.DBusCategory(1, 'foo'), Category('foo', 1)),
    (helpers.DBusCategory(-1, 'foo'), Category('foo', None)),
    (helpers.DBusCategory(-2, ''), None),
))
def test_dbus_to_hamster_category(category_tuple, expectation):
    """Make sure that de-serialization works as intended."""
    result = helpers.dbus_to_hamster_category(category_tuple)
    assert result == expectation
    if expectation:
        assert isinstance(result, Category)


@pytest.mark.parametrize(('activity', 'expectation'), (
    (Activity('foo', pk=1), DBusActivity(1, 'foo', DBusCategory(-2, ''), False)),
    (Activity('foo', pk=None), DBusActivity(-1, 'foo', DBusCategory(-2, ''), False)),
    (None, DBusActivity(-2, '', DBusCategory(-2, ''), False)),
))
def test_hamster_to_dbus_activity(activity, expectation):
    """Make sure that serialization works as intended."""
    result = helpers.hamster_to_dbus_activity(activity)
    assert result == expectation
    assert isinstance(result, DBusActivity)


@pytest.mark.parametrize(('activity_tuple', 'expectation'), (
    (DBusActivity(1, 'foo', DBusCategory(1, 'bar'), False),
     Activity('foo', 1, Category('bar', 1))),
    (DBusActivity(1, 'foo', DBusCategory(-2, ''), False), Activity('foo', 1)),
    (DBusActivity(-2, 'foo', DBusCategory(-2, ''), False), None),
))
def test_dbus_to_hamster_activity(activity_tuple, expectation):
    """Make sure that de-serialization works as intended."""
    result = helpers.dbus_to_hamster_activity(activity_tuple)
    assert result == expectation
    if expectation:
        assert isinstance(result, Activity)


@pytest.mark.parametrize(('tag', 'expectation'), (
    (Tag('foo', 1), helpers.DBusTag(1, 'foo')),
    (Tag('foo', None), helpers.DBusTag(-1, 'foo')),
    (None, helpers.DBusTag(-2, '')),
))
def test_hamster_to_dbus_tag(tag, expectation):
    """Make sure that serialization works as intended."""
    result = helpers.hamster_to_dbus_tag(tag)
    assert result == expectation
    assert isinstance(result, DBusTag)


@pytest.mark.parametrize(('tag_tuple', 'expectation'), (
    ((1, 'foo'), Tag('foo', 1)),
    ((-1, 'foo'), Tag('foo', None)),
    ((-2, ''), None),
    (helpers.DBusTag(1, 'foo'), Tag('foo', 1)),
    (helpers.DBusTag(-1, 'foo'), Tag('foo', None)),
    (helpers.DBusTag(-2, ''), None),
))
def test_dbus_to_hamster_tag(tag_tuple, expectation):
    """Make sure that de-serialization works as intended."""
    result = helpers.dbus_to_hamster_tag(tag_tuple)
    assert result == expectation
    if expectation:
        assert isinstance(result, Tag)


@pytest.mark.parametrize(('fact', 'expectation'), (
    (Fact(Activity('foo', pk=1), dt.datetime(2017, 2, 1, 18), pk=1),
     DBusFact(1, '2017-02-01 18:00:00', '', '',
        DBusActivity(1, 'foo', DBusCategory(-2, ''), False),
        dbus.Array([], '(is)'))),
    (Fact(Activity('foo', pk=1), dt.datetime(2017, 2, 1, 18), pk=None),
     DBusFact(-1, '2017-02-01 18:00:00', '', '',
        DBusActivity(1, 'foo', DBusCategory(-2, ''), False),
        dbus.Array([], '(is)'))),
    (Fact(Activity('foo', pk=1), start=None, pk=None),
     DBusFact(-1, '', '', '',
        DBusActivity(1, 'foo', DBusCategory(-2, ''), False),
        dbus.Array([], '(is)'))),
))
def test_hamster_to_dbus_fact(fact, expectation):
    """Make sure that serialization works as intended."""
    result = helpers.hamster_to_dbus_fact(fact)
    assert result == expectation
    assert isinstance(result, DBusFact)


@pytest.mark.parametrize(('fact_tuple', 'expectation'), (
    (DBusFact(-1, '2017-02-01 18:00:00', '', '',
        DBusActivity(1, 'foo', DBusCategory(-2, ''), False),
        dbus.Array([], '(is)')),
    Fact(Activity('foo', pk=1), dt.datetime(2017, 2, 1, 18), pk=None)),
    (DBusFact(1, '2017-02-01 18:00:00', '', '',
        DBusActivity(1, 'foo', DBusCategory(-2, ''), False),
        dbus.Array([], '(is)')),
    Fact(Activity('foo', pk=1), dt.datetime(2017, 2, 1, 18), pk=1)),
))
def test_dbus_to_hamster_fact(fact_tuple, expectation):
    """Make sure that de-serialization works as intended."""
    result = helpers.dbus_to_hamster_fact(fact_tuple)
    assert result == expectation
    assert isinstance(result, Fact)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Provide a trivial service implementation to export ``hamster-lib`` functionality."""

# This file is part of 'hamster-dbus'.
#
# 'hamster-dbus' is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# 'hamster-dbus' is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  'hamster-dbus'.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, unicode_literals

import datetime
import sys

import hamster_lib
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib

from hamster_dbus import objects


def _get_config():
    """
    Get config to be passed to controller.

    Attention! You need to adjust these values to you own needs! In particular
    ``db_path`` is something you most likly do not want. If you keep it as is
    all changes will be lost on reboot!
    """
    return {
        'store': 'sqlalchemy',
        'day_start': datetime.time(5, 30, 0),
        'fact_min_delta': 60,
        'tmpfile_path': '/tmp/tmpfile.pickle',
        'db_engine': 'sqlite',
        'db_path': ':memory:',
    }


def _main():
    controller = hamster_lib.HamsterControl(_get_config())
    DBusGMainLoop(set_as_default=True)
    loop = GLib.MainLoop()
    main_object = objects.HamsterDBus(loop)
    objects.CategoryManager(controller, main_object)
    objects.ActivityManager(controller, main_object)
    objects.TagManager(controller, main_object)
    objects.FactManager(controller, main_object)
    # Run needs to be called after we setup our service
    loop.run()


if __name__ == '__main__':
    arg = ""
    if len(sys.argv) > 1:
        # truncate args
        arg = sys.argv[1]

    if arg == "server":
        _main()



"""Factories to provide easy to use randomized instances of our main objects."""

import datetime

import factory
import fauxfactory
import hamster_lib


class CategoryFactory(factory.Factory):
    """Provide a factory for randomized ``hamster_lib.Category`` instances."""

    pk = None
    name = fauxfactory.gen_utf8()

    class Meta:
        model = hamster_lib.Category


class TagFactory(factory.Factory):
    """Provide a factory for randomized ``hamster_lib.Tag`` instances."""

    pk = None
    name = fauxfactory.gen_utf8()

    class Meta:
        model = hamster_lib.Tag


class ActivityFactory(factory.Factory):
    """Provide a factory for randomized ``hamster_lib.Activity`` instances."""

    pk = None
    name = fauxfactory.gen_utf8()
    category = factory.SubFactory(CategoryFactory)
    deleted = False

    class Meta:
        model = hamster_lib.Activity


class FactFactory(factory.Factory):
    """Provide a factory for randomized ``hamster_lib.Fact`` instances."""

    pk = None
    activity = factory.SubFactory(ActivityFactory)
    start = factory.Faker('date_time')
    end = factory.LazyAttribute(lambda o: o.start + datetime.timedelta(
        hours=3))
    description = factory.Faker('paragraph')

    class Meta:
        model = hamster_lib.Fact



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of 'hamster-dbus'.
#
# 'hamster-dbus' is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# 'hamster-dbus' is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  'hamster-dbus'.  If not, see <http://www.gnu.org/licenses/>.

"""
A basic dbus service that exposes a subset up hamster-lib.

Most of the methods exposed here are just 'proxies' for the corresponding
``hamster-lib.storage``-manager methods. For details on arguments and semantics
please also refer to their documentation as we want to avoid duplication).

Capitalization violates PEP8 due to dbus specifications on method names
(https://dbus.freedesktop.org/doc/dbus-api-design.html).
"""
# Currently hamster-lib objects need to be converted into something that can be
# passed over dbus. Future iteration should revisit this and see if we can
# expose those objects as dbus objects and consequently just pass those.

# If the ``dbus.service.method`` decorator spans multiple lines we need to add
# the ``   # NOQA`` afterwards instead of the actual function definition.
from __future__ import absolute_import, unicode_literals

import dbus
import dbus.service
import hamster_lib

from hamster_dbus import helpers

DBUS_CATEGORIES_INTERFACE = 'org.projecthamster.HamsterDBus.CategoryManager1'
DBUS_TAGS_INTERFACE = 'org.projecthamster.HamsterDBus.TagManager1'
DBUS_ACTIVITIES_INTERFACE = 'org.projecthamster.HamsterDBus.ActivityManager1'
DBUS_FACTS_INTERFACE = 'org.projecthamster.HamsterDBus.FactManager1'


def _get_dbus_bus_name(bus=None):
    """Return the bus name."""
    # We wrap this in a function instead of a constant to avoid instant
    # instantiation if we use this module as a library.
    if not bus:
        bus = dbus.SessionBus()
    return dbus.service.BusName(
        name='org.projecthamster.HamsterDBus',
        bus=bus
    )


class HamsterDBus(dbus.service.Object):
    """A dbus object providing access to general hamster-lib capabilities."""

    def __init__(self, loop):
        """Initialize main DBus object."""
        self._loop = loop

        super(HamsterDBus, self).__init__(
            bus_name=_get_dbus_bus_name(),
            object_path='/org/projecthamster/HamsterDBus',
        )

    @dbus.service.signal('org.projecthamster.HamsterDBus1')
    def CategoryChanged(self):  # NOQA
        """Signal indicating that at least one category may have been modified."""
        pass

    @dbus.service.signal('org.projecthamster.HamsterDBus1')
    def ActivityChanged(self):  # NOQA
        """Signal indicating that at least one activity may have been modified."""
        pass

    @dbus.service.signal('org.projecthamster.HamsterDBus1')
    def TagChanged(self):  # NOQA
        """Signal indicating that at least one tag may have been modified."""
        pass

    @dbus.service.signal('org.projecthamster.HamsterDBus1')
    def FactChanged(self):  # NOQA
        """Signal indicating that at least one fact may have been modified."""
        pass

    @dbus.service.method('org.projecthamster.HamsterDBus1')
    def Quit(self):  # NOQA
        """Shutdown the service."""
        self._loop.quit()


class CategoryManager(dbus.service.Object):
    """CategoryManager object to be exposed via DBus."""

    def __init__(self, controller, main_object, bus=None):
        """
        Initialize category manager object.

        Args:
            controller: FIXME
            main_object: ``HamsterDBus`` object. This is needed in order to
                emmit signals.
            bus (optional): FIXME
        """
        self._controller = controller
        self._main_object = main_object
        self._busname = _get_dbus_bus_name(bus)

        super(CategoryManager, self).__init__(
            bus_name=self._busname,
            object_path='/org/projecthamster/HamsterDBus/CategoryManager',
        )

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, in_signature='(is)', out_signature='(is)')
    def Save(self, category_tuple):  # NOQA
        """
        Save category.

        Args:
            category_tuple: hamster_lib.Category tuple.

        Returns:
            tuple: ``(category.pk, category.name)`` tuple.
        """
        category = helpers.dbus_to_hamster_category(category_tuple)
        category = self._controller.store.categories.save(category)

        self._main_object.CategoryChanged()
        return helpers.hamster_to_dbus_category(category)

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, in_signature='(is)', out_signature='(is)')
    def GetOrCreate(self, category_tuple):  # NOQA
        """
        For details please refer to ``hamster_lib.storage``.

        Args:
            category (helpers.DBusCategory or None): The category 'dbus encoded'.

        Returns:
            helpers.DBusCategory or None: The retrieved or created category.
                Either way, the returned Category will contain all data from
                the backend, including its primary key.
        """
        category = helpers.dbus_to_hamster_category(category_tuple)
        category = self._controller.store.categories.get_or_create(category)

        self._main_object.CategoryChanged()
        return helpers.hamster_to_dbus_category(category)

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, in_signature='i')
    def Remove(self, pk):  # NOQA
        """
        Remove a category.

        Args:
            pk (int): PK of the category to be removed.

        Returns:
            None: Nothing.
        """
        # [TODO]
        # Once LIB-239 has been solved, we should be able to skip extra category
        # retrieval.
        category = self._controller.store.categories.get(pk)
        self._controller.store.categories.remove(category)

        self._main_object.CategoryChanged()
        return None

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, in_signature='i', out_signature='(is)')
    def Get(self, pk):  # NOQA
        """
        Return a category based on their pk.

        Args:
            pk (int): PK of the category to be retrieved.

        Returns:
            tuple: helpers.DBusCategory tuple.

        Raises:
            KeyError: If no such PK was found.
        """
        category = self._controller.categories.get(pk)
        return helpers.hamster_to_dbus_category(category)

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, in_signature='s', out_signature='(is)')
    def GetByName(self, name):  # NOQA
        """
        Look up a category by its name and return its PK.

        Args:
            name (str): Name of the category to we want the PK of.

        Returns:
            tuple: hamster_lib.Category tuple.
        """
        category = self._controller.categories.get_by_name(name)
        return helpers.hamster_to_dbus_category(category)

    @dbus.service.method(DBUS_CATEGORIES_INTERFACE, out_signature='a(is)')
    def GetAll(self):  # NOQA
        """
        Get all categories.

        Returns:
            list: List of tuples with (category.pk, category.name)
        """
        categories = self._controller.categories.get_all()
        return [helpers.hamster_to_dbus_category(category) for category in categories]


class TagManager(dbus.service.Object):
    """TagManager object to be exposed via DBus."""

    def __init__(self, controller, main_object, bus=None):
        """
        Initialize tag manager object.

        Args:
            controler: FIXME
            main_object: ``HamsterDBus`` object. This is needed in order to
                emmit signals.
            bus (optional): FIXME
        """
        self._controller = controller
        self._main_object = main_object
        self._busname = _get_dbus_bus_name(bus)

        super(TagManager, self).__init__(
            bus_name=_get_dbus_bus_name(),
            object_path='/org/projecthamster/HamsterDBus/TagManager',
        )

    @dbus.service.method(DBUS_TAGS_INTERFACE, in_signature='(is)', out_signature='(is)')
    def Save(self, tag_tuple):  # NOQA
        """
        Save tag.

        Args:
            tag_tuple: hamster_lib.Tag tuple.

        Returns:
            helpers.DBusTag: For details please see ``helpers.haster_to_dbus_tag``.
        """
        tag = helpers.dbus_to_hamster_tag(tag_tuple)
        tag = self._controller.store.tags.save(tag)

        self._main_object.TagChanged()
        return helpers.hamster_to_dbus_tag(tag)

    @dbus.service.method(DBUS_TAGS_INTERFACE, in_signature='i')
    def Remove(self, pk):  # NOQA
        """
        Remove a tag.

        Args:
            pk (int): PK of the tag to be removed.

        Returns:
            None: Nothing.
        """
        # [TODO]
        # Once LIB-239 has been solved, we should be able to skip extra tag
        # retrieval.
        tag = self._controller.store.tags.get(pk)
        self._controller.store.tags.remove(tag)

        self._main_object.TagChanged()
        return None

    @dbus.service.method(DBUS_TAGS_INTERFACE, in_signature='s', out_signature='(is)')
    def GetByName(self, name):  # NOQA
        """
        Look up a tag by its name and return its PK.

        Args:
            name (str): Name of the tag to we want the PK of.

        Returns:
            helpers.DBusTag: For details please see ``helpers.haster_to_dbus_tag``.
        """
        tag = self._controller.store.tags.get_by_name(name)
        return helpers.hamster_to_dbus_tag(tag)

    @dbus.service.method(DBUS_TAGS_INTERFACE, out_signature='a(is)')
    def GetAll(self):  # NOQA
        """
        Get all tags.

        Returns:
            list: List of ``helpers.DBusTag``s. For details see
                ``helpers.hamster_to_dbus_tag``.
        """
        tags = self._controller.store.tags.get_all()
        return [helpers.hamster_to_dbus_tag(tag) for tag in tags]


class ActivityManager(dbus.service.Object):
    """ActivityManager object to be exposed via DBus."""

    def __init__(self, controller, main_object):
        """
        Initialize activity manager object.

        Args:
            controler: FIXME
            main_object: ``HamsterDBus`` object. This is needed in order to
                emmit signals.
        """
        # [FIXME]
        # Unlike with ``CategoryManager`` and ``TagManager`` we do not allow for
        # a custom bus.
        self._controller = controller
        self._main_object = main_object

        super(ActivityManager, self).__init__(
            bus_name=_get_dbus_bus_name(),
            object_path='/org/projecthamster/HamsterDBus/ActivityManager',
        )

    @dbus.service.method(DBUS_ACTIVITIES_INTERFACE, in_signature='(is(is)b)',
        out_signature='(is(is)b)')  # NOQA
    def Save(self, activity_tuple):
        """
        Save an activity.

        Args:
            activity_tuple (tuple): Tuple with activity values.

        Returns:
            tuple: Tuple representing the saved activity.
        """
        activity = helpers.dbus_to_hamster_activity(activity_tuple)
        result = self._controller.activities.save(activity)

        self._main_object.ActivityChanged()
        self._main_object.CategoryChanged()
        # This is because during activity creation/updating a new category may
        # be created as well.
        return helpers.hamster_to_dbus_activity(result)

    @dbus.service.method(DBUS_ACTIVITIES_INTERFACE, in_signature='i')
    def Remove(self, pk):  # NOQA
        """Remove an activity.

        Args:
            pk (int): PK of the activity to be removed.

        Returns:
            None: Nothing.
        """
        activity = self._controller.activities.get(pk)
        self._controller.activities.remove(activity)

        self._main_object.ActivityChanged()
        return None

    @dbus.service.method(DBUS_ACTIVITIES_INTERFACE, in_signature='i', out_signature='(is(is)b)')
    def Get(self, pk):  # NOQA
        """
        Retrieve an ``hamster_lib.Activity`` based on it's PK.

        Args:
            pk (int): PK of the activity to be fetched.

        Returns:
            tuple: hamster_lib.Activity tuple
        """
        activity = self._controller.store.activities.get(pk)
        return helpers.hamster_to_dbus_activity(activity)

    @dbus.service.method(DBUS_ACTIVITIES_INTERFACE, in_signature='i',
        out_signature='a(is(is)b)')  # NOQA
    def GetAll(self, category_pk):
        """
        Retrieve all ``hamster_lib.Activity`` instances that match the criteria.

        Args:
            category_pk (int): hamster_lib.Category pk. Use ``-1`` for ``None`` and ``-2`` for
                ``False``. Refer to ``hamster_lib.storage.hamster_lib.ActivityManager.get_all``
                for details.

        Returns:
            tuple: (activity_tuple, error).
        """
        if category_pk == -1:
            category = None
        elif category_pk == -2:
            category = False
        else:
            category = self._controller.store.categories.get(category_pk)

        activities = self._controller.store.activities.get_all(category)

        return [helpers.hamster_to_dbus_activity(activity) for activity in activities]


class FactManager(dbus.service.Object):
    """FactManager object to be exposed via DBus."""

    def __init__(self, controller, main_object):
        """
        Initialize fact manager object.

        Args:
            controler: FIXME
            main_object: ``HamsterDBus`` object. This is needed in order to
                emmit signals.
        """
        # [FIXME]
        # Unlike with ``CategoryManager`` and ``TagManager`` we do not allow for
        # a custom bus.
        self._controller = controller
        self._main_object = main_object

        super(FactManager, self).__init__(
            bus_name=_get_dbus_bus_name(),
            object_path='/org/projecthamster/HamsterDBus/FactManager',
        )

    @dbus.service.method(DBUS_FACTS_INTERFACE, in_signature='s',
        out_signature='(isss(is(is)b)a(is))')  # NOQA
    def SaveRaw(self, raw_fact):
        """
        Take a raw_fact save it to our backend.

        Args:
            raw_fact (str): ``raw fact`` string.

        Returns:
            tuple (DBushamster_lib.Fact): Serialized version of the saved ``hamster_lib.Fact``
            instance.

        Note: This method is identical to ``Savehamster_lib.Fact`` with the only difference being
            that it takes a ``raw fact`` instead of a serialized ``hamster_lib.Fact`` instance.
        """
        fact = hamster_lib.Fact.create_from_raw_fact(raw_fact)
        result = self._controller.store.facts.save(fact)

        self._main_object.FactChanged()
        self._main_object.CategoryChanged()
        self._main_object.TagChanged()
        self._main_object.ActivityChanged()
        # This is because during creation/updating a new related instances
        # may be created as well.

        return helpers.hamster_to_dbus_fact(result)

    @dbus.service.method(DBUS_FACTS_INTERFACE, in_signature='(isss(is(is)b)a(is))',
        out_signature='(isss(is(is)b)a(is))')  # NOQA
    def Save(self, fact_tuple):
        """
        Take a fact save it to our backend.

        Args:
            fact_tuple (DBushamster_lib.Fact): ``hamster_lib.Fact`` to be saved.

        Returns:
            tuple (DBusFact): Serialized version of the saved ``hamster_lib.Fact``
                instance.
        """
        fact = helpers.dbus_to_hamster_fact(fact_tuple)
        result = self._controller.store.facts.save(fact)

        self._main_object.FactChanged()
        self._main_object.CategoryChanged()
        self._main_object.TagChanged()
        self._main_object.ActivityChanged()
        # This is because during creation/updating a new related instances
        # may be created as well.

        return helpers.hamster_to_dbus_fact(result)

    @dbus.service.method(DBUS_FACTS_INTERFACE, in_signature='i')
    def Remove(self, pk):  # NOQA
        """
        Remove fact from storage by it's PK.

        Args:
            fact_pk (int): PK of the fact to be removed.

        Returns:
            None: Nothing.
        """
        fact = self._controller.store.facts.get(pk)
        self._controller.store.facts.remove(fact)

        self._main_object.FactChanged()
        return None

    @dbus.service.method(DBUS_FACTS_INTERFACE, in_signature='i',
        out_signature='(isss(is(is)b)a(is))')  # NOQA
    def Get(self, fact_pk):
        """Get fact by PK.

        Args:
            fact_pk (int): PK of the fact to be retrieved.

        Returns:
            DBushamster_lib.Fact: Serialized ``hamster_lib.Fact`` instance.
        """
        fact = self._controller.facts.get(fact_pk)
        return helpers.hamster_to_dbus_fact(fact)

    @dbus.service.method(DBUS_FACTS_INTERFACE, in_signature='sss',
        out_signature='a(isss(is(is)b)a(is))')  # NOQA
    def GetAll(self, start, end, filter_term):
        """
        Get all facts matching criteria.

        Args:
            start (int): Unix-timestamp for start of timeframe. ``-1`` for ``None``.
            end (int): Unix-timestamp for end of timeframe. ``-1`` for ``None``.
            filter_term (str): Only consider ``hamster_lib.Facts`` with this string as part of
                their associated ``hamster_lib.Activity.name``

        Returns:
            list: A list of ``helpers.DBushamster_lib.Fact``-tuples.
                For details on those, please see ``helpers.hamster_to_dbus_fact``.

        Note:
            Our ``hamsterlib`` manager method is actually more flexible and would allow
            for ``datetime.datetime`` or ``datetime.time`` instances instead of just
            ``datetime.date`` instances for ``start`` and ``end``. But for compatibility
            reasons we stick with the simpler legacy version for now.
        """
        def get_start(start):
            return None

        def get_end(end):
            return None

        def get_filter_term(end):
            return filter_term

        facts = self._controller.store.facts.get_all(get_start(start), get_end(end),
            get_filter_term(filter_term))
        return [helpers.hamster_to_dbus_fact(fact) for fact in facts]

    @dbus.service.method(DBUS_FACTS_INTERFACE, out_signature='a(isss(is(is)b)a(is))')
    def GetTodays(self):  # NOQA
        """
        Get facts of today, respecting hamster day_start, day_end settings.

        Returns:
            list: A list of ``helpers.DBushamster_lib.Fact``-tuples.
                For details on those, please see ``helpers.DBushamster_lib.Fact``.

        Note:
            This only returns proper facts and will not include any ongoing fact!
        """
        facts = self.controller.store.facts.get_today()
        return [helpers.dbus_to_hamster_fact(fact) for fact in facts]



# -*- coding: utf-8 -*-

"""
Unittests for ``hamster_dbus.storage.FactManager``.

Please refer to ``__init__.py`` for general details.
"""

from __future__ import absolute_import, unicode_literals

import datetime
import subprocess

import dbus
import dbusmock
from hamster_lib import objects as lib_objects

from hamster_dbus import storage

from . import common
from .. import factories


class BaseTestFactManager(common.HamsterDBusManagerTestCase):
    """Base test case that provides infrastructure common to all other test cases."""

    def setUp(self):
        """
        Setup a mock ``FactManager`` object and provide a convenient interface.

        By adding methods to ``self.interface`` individual tests can mock an available
        dbus object while ``self.manager`` provides easy access to an actual
        ``FactManager`` instance. By explicitly passing our test session bus
        we make sure the manager queries against the right bus/object.
        """
        self.service_mock = self.spawn_server(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/FactManager',
            'org.projecthamster.HamsterDBus.FactManager1',
            stdout=subprocess.PIPE
        )

        self.dbus_object = self.dbus_con.get_object(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/FactManager'
        )

        self.interface = dbus.Interface(self.dbus_object, dbusmock.MOCK_IFACE)
        self.manager = storage.FactManager(bus=self.dbus_con)


class TestSave(BaseTestFactManager):

    def setUp(self):
        """Test setup."""
        super(TestSave, self).setUp()
        self.new_fact = factories.FactFactory()
        self.existing_fact = factories.FactFactory(pk=1)

    def test_save_existing(self):
        """Make sure a ``Fact`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'Save', '(isss(is(is)b)a(is))', '(isss(is(is)b)a(is))',
            'ret = (1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])'
        )

        result = self.manager.save(self.existing_fact)
        self.assertIsInstance(result, lib_objects.Fact)

    def test_save_new(self):
        """Make sure a ``Fact`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'Save', '(isss(is(is)b)a(is))', '(isss(is(is)b)a(is))',
            'ret = (1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])'
        )

        result = self.manager.save(self.new_fact)
        self.assertIsInstance(result, lib_objects.Fact)

    def test_non_fact_instance(self):
        """Make sure that passing anything but a ``Fact`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.save('foobar')


class TestRemove(BaseTestFactManager):

    def setUp(self):
        """Test setup."""
        super(TestRemove, self).setUp()
        self.existing_fact = factories.FactFactory(pk=1)

    def test_remove(self):
        """Make sure ``None`` is returned."""
        self.dbus_object.AddMethod('', 'Remove', 'i', '', '')

        result = self.manager.remove(self.existing_fact)
        self.assertIsNone(result)

    def test_remove_non_fact(self):
        """Make sure that passing anything but a ``Fact`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.remove(1)


class TestGet(BaseTestFactManager):

    def setUp(self):
        """Test setup."""
        super(TestGet, self).setUp()
        self.existing_fact = factories.FactFactory(pk=1)

    def test_get(self):
        """Make sure a ``Fact`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'Get', 'i', '(isss(is(is)b)a(is))',
            'ret = (1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])'
        )

        result = self.manager.get(self.existing_fact.pk)
        self.assertIsInstance(result, lib_objects.Fact)


class TestGetAll(BaseTestFactManager):

    def test_get_all(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod(
            '', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))',
            'ret = [(1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])]'
        )

        result = self.manager.get_all()
        for each in result:
            self.assertIsInstance(each, lib_objects.Fact)

    def test_start_datetime(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(start=datetime.datetime(2017, 2, 1, 17))
        self.assertIsInstance(result, list)

    def test_start_date(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(start=datetime.date(2017, 2, 1))
        self.assertIsInstance(result, list)

    def test_start_time(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(start=datetime.time(17))
        self.assertIsInstance(result, list)

    def test_start_none(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(start=None)
        self.assertIsInstance(result, list)

    def test_invalid_start_type(self):
        """Make sure that passing an invalid ``start`` argument throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_all(start='2012-02-01 13:30')

    def test_end_datetime(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(end=datetime.datetime(2017, 2, 1, 17))
        self.assertIsInstance(result, list)

    def test_end_date(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(end=datetime.date(2017, 2, 1))
        self.assertIsInstance(result, list)

    def test_end_time(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(end=datetime.time(17))
        self.assertIsInstance(result, list)

    def test_end_none(self):
        """Make sure a iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', 'sss', 'a(isss(is(is)b)a(is))', 'ret = []')

        result = self.manager.get_all(end=None)
        self.assertIsInstance(result, list)

    def test_invalid_end_type(self):
        """Make sure that passing an invalid ``end`` argument throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_all(end='2012-02-01 13:30')

    def test_end_before_start(self):
        """Make sure that passing an ``end<start`` throws an error."""
        with self.assertRaises(ValueError):
            self.manager.get_all(
                start=datetime.datetime(2017, 2, 2, 18),
                end=datetime.datetime(2017, 2, 1, 18)
            )


class TestGetToday(BaseTestFactManager):

    def test_get(self):
        """Make sure a  iterator of ``Fact`` instances is returned."""
        self.dbus_object.AddMethod(
            '', 'GetToday', '', 'a(isss(is(is)b)a(is))',
            'ret = [(1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])]'
        )

        result = self.manager.get_today()
        for each in result:
            self.assertIsInstance(each, lib_objects.Fact)


class TestStopTmpFact(BaseTestFactManager):

    def setUp(self):
        super(TestStopTmpFact, self).setUp()

    def test_stop_tmp_fact(self):
        """Make sure a ``Fact`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'StopTmpFact', '', '(isss(is(is)b)a(is))',
            'ret = (1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])'
        )

        result = self.manager.stop_tmp_fact()
        self.assertIsInstance(result, lib_objects.Fact)


class TestCancelTmpFact(BaseTestFactManager):

    def test_cancel_tmp_fact(self):
        """Make sure ``None`` is returned."""
        self.dbus_object.AddMethod(
            '', 'CancelTmpFact', '', '', '')

        result = self.manager.cancel_tmp_fact()
        self.assertIsNone(result)


class TestGetTmpFact(BaseTestFactManager):

    def test_stop_tmp_fact(self):
        """Make sure a ``Fact`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'GetTmpFact', '', '(isss(is(is)b)a(is))',
            'ret = (1, "2016-12-01 18:00:00", "2016-12-01 19:00:00", "description",'
            '(1, "foo", (2, "bar"), False), [(1, "tag1"), (2, "tag2")])'
        )

        result = self.manager.get_tmp_fact()
        self.assertIsInstance(result, lib_objects.Fact)



# -*- coding: utf-8 -*-

"""
Unittests for DBusStore.

Please refer to ``__init__.py`` for general details.
"""

from __future__ import absolute_import, unicode_literals

import subprocess

from hamster_dbus import storage

from . import common


class TestDBusStore(common.HamsterDBusManagerTestCase):

    def setUp(self):
        """Setup test environment."""
        # We have to launch a mocked service in order to provide the
        # 'org.projecthamster.HamsterDBus' namespace.
        # Which object and interface does not actually matter for this test.
        self.service_mock = self.spawn_server(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/FactManager',
            'org.projecthamster.HamsterDBus.FactManager1',
            stdout=subprocess.PIPE
        )

        # For our test purpose, an empty config test suffices.
        self.store = storage.DBusStore({})

    def test_categories_manager(self):
        """Make sure a ``storage.CategoryManager`` is instantiated."""
        self.assertIsInstance(self.store.categories, storage.CategoryManager)

    def test_activities_manager(self):
        """Make sure a ``storage.ActivityManager`` is instantiated."""
        self.assertIsInstance(self.store.activities, storage.ActivityManager)

    def test_tags_manager(self):
        """Make sure a ``storage.TagManager`` is instantiated."""
        self.assertIsInstance(self.store.tags, storage.TagManager)

    def test_facts_manager(self):
        """Make sure a ``storage.FactManager`` is instantiated."""
        self.assertIsInstance(self.store.facts, storage.FactManager)

    def test_cleanup(self):
        """Test the cleanup method."""
        self.assertIsNone(self.store.cleanup())

    def test_explicit_bus(self):
        """Make sure that an explicitly passed bus is really used."""
        self.store = storage.DBusStore({}, bus=self.dbus_con)
        self.assertEqual(self.store._bus, self.dbus_con)



# This file is part of 'hamster-dbus'.
#
# 'hamster-dbus' is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# 'hamster-dbus' is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with 'hamster-dbus'.  If not, see <http://www.gnu.org/licenses/>.

"""
Helper functions to ease working with dbus objects and type conversion.

This is the central place were the dbus "signature" of our hamster_lib objects
is defined and documented. All dbus code is advised to use those serialization methods to
ensure consistend representation.
"""

import datetime
from collections import namedtuple

import dbus
import hamster_lib
from six import text_type

DBusCategory = namedtuple('DBusCategory', ('pk', 'name'))
# 'category' is supposed to store an ``DBushamster_lib.Category`` instance.
DBusActivity = namedtuple('DBusActivity', ('pk', 'name', 'category', 'deleted'))
DBusTag = namedtuple('DBusTag', ('pk', 'name'))
# 'activity' is supposed to store an ``DBushamster_lib.Activity`` instance.
DBusFact = namedtuple('DBusFact', ('pk', 'start', 'end', 'description', 'activity', 'tags'))


def _none_to_int(value):
    """
    Serialize ``None`` as an integer value.

    Args:
        value (int, None): Value to 'sanitize'.

    Returns:
        int: Return ``-1`` if ``value=None`` else the original value.
    """
    if value is None:
        value = -1
    return int(value)


def _int_to_none(value):
    """
    Convert integer representation of ``None`` to the proper python type.

    Args:
        value (int): Value to convert to 'sanitize'.

    Return:
        int or None: The original integer or ``None`` if ``value=-1``.
    """
    value = int(value)
    if value == -1:
        value = None
    return value


def datetime_to_text(datetime_info):
    """
    Serialize ``datetime``, ``date`` or ``time`` instances as text.

    Args:
        datetime_info (dt.datetime, dt.date, dt.time, None): Python datetime
        information that is to be serialized.

    Returns:
        text_type: Returns optinionated text representation of passed datetime
        information. ``dt.datetime`` will be represented as ``%Y-%m-%d %H:%M:%S``,
        ``dt.date`` will be represented as ``%Y-%m-%d`` and ``dt.time`` as
        ``%H-%M-%S``. ``None`` will be serialized as an ``empty string``.

    Raises:
        TypeError: If ``datetime_info`` is not ``datetime.datetime``,
        ``datetime.date``, ``datetime.time`` or ``None``.
    """
    if isinstance(datetime_info, datetime.datetime):
        result = datetime_info.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(datetime_info, datetime.date):
        result = datetime_info.strftime('%Y-%m-%d')
    elif isinstance(datetime_info, datetime.time):
        result = datetime_info.strftime('%H:%M:%S')
    elif datetime_info is None:
        result = ''
    else:
        raise TypeError
    return text_type(result)


def text_to_datetime(datetime_text):
    """
    Serialize a text string to its corresponding python datetime instance.

    Args:
        datetime_text (text_type): Text to be converted. If not an
        ``empty string`` this needs to match one of the following formats:
        ``%Y-%m-%d %H:%M:%S``, ``%Y-%m-%d`` or ``%H:%M:%S``,

    Returns:
        datetime.datetime, datetime.date, datetime.time: A python instance
        mathing the supplied text or the ``empty string`` if
        ``datetime_text=None``.

    Raises:
        ValueError: If passed text does not match any of the given formats.

    """
    if datetime_text == '':
        result = None
    else:
        try:
            result = datetime.datetime.strptime(datetime_text, '%Y-%m-%d %H:%M:%S')
        except ValueError:
            try:
                result = datetime.datetime.strptime(datetime_text, '%Y-%m-%d').date()
            except ValueError:
                result = datetime.datetime.strptime(datetime_text, '%H:%M:%S').time()
    return result


# This is needed because not all types used in ``as_tuple`` can be passed
# through dbus
def hamster_to_dbus_category(category):
    """
    Convert a ``hamster_lib.Category`` instance to its dbus representation.

    The resulting tuple is suitable for dbus messages and has the following
    signature: '(is)'.

    i = category.pk
    s = category.name

    Args:
        category (hamster_lib.Category): Category instance to be serialized.

    Returns:
        DBusCategory: Serialized category instance. If the category to be
        serialized does not yet have a PK the resulting tuple will have
        ``pk=-1``. If the category is actually ``None`` the resulting
        tuple will have ``pk=-2``.
    """
    def get_pk(category):
        return _none_to_int(category.pk)

    def get_name(category):
        return category.name

    if category is None:
        result = DBusCategory(-2, '')
    else:
        result = DBusCategory(
            pk=get_pk(category),
            name=get_name(category)
        )
    return result


def dbus_to_hamster_category(category_tuple):
    """
    Return a ``hamster_lib.Category`` from its passed DBus representation.

    Args:
        category_tuple (DBusCategory or tuple): Tuple to be serialized. If a
        non ``DBusCategory`` tuple is passed its values need to be ordered as
        for ``DBusCategory``.
        A ``category.pk`` of ``-1`` will be converted to ``None``.

    Returns:
        hamster_lib.Category or None: Category constructed from tuple data.
        If ``category_tuple.pk=-2`` ``None`` will be returned.
    """
    def get_pk(category_tuple):
        pk = category_tuple.pk
        return _int_to_none(pk)

    def get_name(category_tuple):
        return category_tuple.name

    category_tuple = DBusCategory(*category_tuple)

    if category_tuple.pk == -2:
        result = None
    else:
        result = hamster_lib.Category(
            pk=get_pk(category_tuple),
            name=get_name(category_tuple)
        )
    return result


def hamster_to_dbus_activity(activity):
    """
    Convert a ``hamster_lib.Activity`` instance to it's dbus representation.

    The resulting tuple is suitable for dbus messages and has the following
    signature: '(is(is))'. For details on category serialization see
    ``hamster_to_dbus_category``.

    i = activity.pk
    s = activity.name
    (is) = category tuple: (category.pk, category.name)

    Args:
        activity (hamster_lib.Activity): Activity instance to be serialized.

    Returns:
        DBusActivity: Serialized activity instance. If the activity to be
        serialized does not yet have a PK the resulting tuple will have
        ``pk=-1``. If the activity is actually ``None`` the resulting
        tuple will have ``pk=-2``.
    """
    def get_pk(activity):
        pk = activity.pk
        return _none_to_int(pk)

    def get_name(activity):
        return activity.name

    def get_category(activity):
        return hamster_to_dbus_category(activity.category)

    def get_deleted(activity):
        return activity.deleted

    if activity is None:
        result = DBusActivity(-2, '', DBusCategory(-2, ''), False)
    else:
        result = DBusActivity(
            pk=get_pk(activity),
            name=get_name(activity),
            category=get_category(activity),
            deleted=get_deleted(activity)
        )
    return result


def dbus_to_hamster_activity(activity_tuple):
    """
    Convert an 'DBus activity tuple to activity.

    Args:
        activity_tuple (DBusActivity or tuple): Tuple to be serialized. If a
        non ``DBusActivity`` tuple is passed its values need to be ordered as
        for ``DBusActivity``.
        An ``activity.pk`` of ``-1`` will be converted to ``None``.

    Returns:
        hamster_lib.Activity or None: hamster_lib.Activity instance. If
        ``activity_tuple.pk=-2`` ``None`` will be returned.
    """
    def get_pk(activity_tuple):
        return _int_to_none(activity_tuple.pk)

    def get_name(activity_tuple):
        return activity_tuple.name

    def get_category(activity_tuple):
        category_tuple = activity_tuple.category
        return dbus_to_hamster_category(category_tuple)

    activity_tuple = DBusActivity(*activity_tuple)

    if activity_tuple.pk == -2:
        result = None
    else:
        result = hamster_lib.Activity(
            pk=get_pk(activity_tuple),
            name=get_name(activity_tuple),
            category=get_category(activity_tuple),
        )
    return result


def hamster_to_dbus_tag(tag):
    """Convert a ``hamster_lib.Tag`` instance to its dbus representation.

    The resulting tuple is suitable for dbus messages and has the following
    signature: '(is)'.

    i = tag.pk
    s = tag.name

    Args:
        tag (hamster_lib.Tag): Tag instance to be serialized.

    Returns:
        DBusTag: Serialized tag instance. If the tag to be
        serialized does not yet have a PK the resulting tuple will have
        ``pk=-1``. If the tag is actually ``None`` the resulting
        tuple will have ``pk=-2``.
    """
    def get_pk(tag):
        return _none_to_int(tag.pk)

    def get_name(tag):
        return tag.name

    if tag is None:
        result = DBusTag(-2, '')
    else:
        result = DBusTag(
            pk=get_pk(tag),
            name=get_name(tag)
        )
    return result


def dbus_to_hamster_tag(tag_tuple):
    """Return a ``hamster_lib.Tag`` from its passed dbus representation.

    Args:
        tag_tuple (DBusTag or tuple): Tuple to be serialized. If a
        non ``DBusTag`` tuple is passed its values need to be ordered as
        for ``DBusTag``.
        A ``tag.pk`` of ``-1`` will be converted to ``None``.

    Returns:
        hamster_lib.Tag or None: Tag constructed from tuple data.
        If ``tag_tuple.pk=-2`` ``None`` will be returned.
    """
    def get_pk(tag_tuple):
        pk = tag_tuple.pk
        return _int_to_none(pk)

    def get_name(tag_tuple):
        return tag_tuple.name

    tag_tuple = DBusTag(*tag_tuple)

    if tag_tuple.pk == -2:
        result = None
    else:
        result = hamster_lib.Tag(
            pk=get_pk(tag_tuple),
            name=get_name(tag_tuple)
        )
    return result


# Note that we currently do not support passing timezone information!
def hamster_to_dbus_fact(fact):
    """
    Perform the conversion between hamster_lib.Factinstance and dbus supported data types.

    The resulting tuple is suitable for dbus messages and has the following
    signature: (isss(is(is)b)a(is))'

    i           pk
    s           start
    s           end
    s           description
    (is(is)b)   activity tuple: (pk, name, category_tuple)
    a(is)       list of tag tuples: (pk, name)

    Args:
        fact (hamster_lib.Fact): Fact instance to be serialized.

    Returns:
        DBusFact: Serialized fact instance. If the  fact to be
        serialized does not yet have a PK the resulting tuple will have
        ``pk=-1``. For details on the serialized activity refer to
        ``hamster_to_dbus_activity``.
    """
    def get_pk(fact):
        return _none_to_int(fact.pk)

    def get_start(fact):
        return datetime_to_text(fact.start)

    def get_end(fact):
        return datetime_to_text(fact.end)

    def get_activity(fact):
        activity = fact.activity
        return hamster_to_dbus_activity(activity)

    def get_description(fact):
        result = fact.description
        if fact.description is None:
            result = ''
        return result

    def get_tags(fact):
        # We need to build the Array explicitly in order to avoid python-dbus
        # trying to guess its signature (which fails for empty lists).
        return dbus.Array([hamster_to_dbus_tag(tag) for tag in fact.tags], '(is)')

    return DBusFact(
        pk=get_pk(fact),
        start=get_start(fact),
        end=get_end(fact),
        description=get_description(fact),
        activity=get_activity(fact),
        tags=get_tags(fact),
    )


def dbus_to_hamster_fact(fact_tuple):
    """
    Return a ``hamster_lib.Fact`` instance from its dbus representation.

    Args:
        fact_tuple (DBusFact or tuple): Tuple to be serialized. If a
        non ``DBusFact`` tuple is passed its values need to be ordered as
        for ``DBusFact``.
        A ``fact.pk`` of ``-1`` will be converted to ``None``.

    Returns:
        hamster_lib.Fact: Fact constructed from tuple data.
    """
    def get_pk(fact_tuple):
        return _int_to_none(fact_tuple.pk)

    def get_start(fact_tuple):
        return text_to_datetime(fact_tuple.start)

    def get_end(fact_tuple):
        return text_to_datetime(fact_tuple.end)

    def get_activity(fact_tuple):
        return dbus_to_hamster_activity(fact_tuple.activity)

    def get_tags(fact_tuple):
        return [dbus_to_hamster_tag(tag) for tag in fact_tuple.tags]

    def get_description(fact_tuple):
        return fact_tuple.description

    # Make sure we can deal with proper DBusFact instances as well as with
    # 'unprepared' normal tuples.
    fact_tuple = DBusFact(*fact_tuple)
    return hamster_lib.Fact(
        pk=get_pk(fact_tuple),
        start=get_start(fact_tuple),
        end=get_end(fact_tuple),
        activity=get_activity(fact_tuple),
        tags=get_tags(fact_tuple),
        description=get_description(fact_tuple)
    )



# -*- coding: utf-8 -*-

"""Integration tests for hamster_dbus.objects."""

import pytest

import hamster_dbus.helpers as helpers


@pytest.mark.needs_dbus_service
class TestCategoryManager(object):

    def test_save_new(self, category_manager, category, category_name_parametrized):
        """Make sure an instance is created and returned."""

        category.name = category_name_parametrized
        dbus_category = helpers.hamster_to_dbus_category(category)
        result = category_manager.Save(dbus_category)
        result = helpers.dbus_to_hamster_category(result)
        assert result.pk
        assert category.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_get_or_create_created(self, category_manager, category):
        """Make sure an instance is created and returned."""
        assert not category_manager.GetAll()
        dbus_category = helpers.hamster_to_dbus_category(category)
        result = category_manager.GetOrCreate(dbus_category)
        result = helpers.dbus_to_hamster_category(result)
        assert result.pk
        assert category.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_get_or_create_get(self, category_manager, stored_category):
        """If name exists make sure it is fetched and no new one created."""
        assert len(category_manager.GetAll()) == 1
        dbus_category = helpers.hamster_to_dbus_category(stored_category)
        result = category_manager.GetOrCreate(dbus_category)
        result = helpers.dbus_to_hamster_category(result)
        assert len(category_manager.GetAll()) == 1
        assert stored_category.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_remove(self, category_manager, stored_category):
        """Make sure a category is removed."""
        result = category_manager.Remove(stored_category.pk)
        assert result is None
        # [FIXME]
        # with pytest.raises(KeyError):
        #    store.categories.get(stored_category.pk)

    def test_get(self, category_manager, stored_category):
        """Make sure a matching category is returned."""
        result = category_manager.Get(stored_category.pk)
        result = helpers.dbus_to_hamster_category(result)
        assert result.pk == stored_category.pk
        assert result.name == stored_category.name

    def test_get_by_name(self, category_manager, stored_category):
        """Make sure a matching category is returned."""
        result = category_manager.GetByName(stored_category.name)
        result = helpers.dbus_to_hamster_category(result)
        assert result.pk == stored_category.pk
        assert result.name == stored_category.name

    def test_get_all(self, category_manager, stored_category_batch_factory):
        """Make sure we get all stored categories."""
        categories = stored_category_batch_factory(5)
        result = category_manager.GetAll()
        result = [helpers.dbus_to_hamster_category(each) for each in result]
        assert len(result) == 5
        for category in categories:
            assert category in result


@pytest.mark.needs_dbus_service
class TestActivityManager(object):

    def test_save_new(self, activity_manager, activity):
        """Make sure instance is saved and returned."""
        dbus_activity = helpers.hamster_to_dbus_activity(activity)
        result = activity_manager.Save(dbus_activity)
        result = helpers.dbus_to_hamster_activity(result)
        assert result.pk
        assert result.name == activity.name
        # We create a new instance, hence result will have a pk where the
        # original does not.
        assert activity.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_remove(self, activity_manager, stored_activity):
        """Make sure instance is removed."""
        result = activity_manager.Remove(stored_activity.pk)
        assert result is None
        # [FIXME]
        # with pytest.raises(KeyError):
        #     store.activities.get(stored_activity.pk)

    def test_get(self, activity_manager, stored_activity):
        """Make sure instance is returned."""
        result = activity_manager.Get(stored_activity.pk)
        result = helpers.dbus_to_hamster_activity(result)
        assert result == stored_activity

    def test_get_all(self, activity_manager, stored_activity_batch_factory):
        """Make sure we get all stored categories."""
        activities = stored_activity_batch_factory(5)
        result = activity_manager.GetAll(-2)
        result = [helpers.dbus_to_hamster_activity(each) for each in result]
        assert len(result) == 5
        for activity in activities:
            assert activity in result


@pytest.mark.needs_dbus_service
class TestTagManager(object):

    def test_save_new(self, tag_manager, tag, tag_name_parametrized):
        """Make sure an instance is created and returned."""

        tag.name = tag_name_parametrized
        dbus_tag = helpers.hamster_to_dbus_tag(tag)
        result = tag_manager.Save(dbus_tag)
        result = helpers.dbus_to_hamster_tag(result)
        assert result.pk
        assert tag.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_remove(self, tag_manager, stored_tag):
        """Make sure a tag is removed."""
        result = tag_manager.Remove(stored_tag.pk)
        assert result is None
        # [FIXME]
        # with pytest.raises(KeyError):
        #    store.categories.get(stored_tag.pk)

    def test_get_by_name(self, tag_manager, stored_tag):
        """Make sure a matching tag is returned."""
        result = tag_manager.GetByName(stored_tag.name)
        result = helpers.dbus_to_hamster_tag(result)
        assert result.pk == stored_tag.pk
        assert result.name == stored_tag.name

    def test_get_all(self, tag_manager, stored_tag_batch_factory):
        """Make sure we get all stored categories."""
        tags = stored_tag_batch_factory(5)
        result = tag_manager.GetAll()
        result = [helpers.dbus_to_hamster_tag(each) for each in result]
        assert len(result) == 5
        for tag in tags:
            assert tag in result


@pytest.mark.needs_dbus_service
class TestFactManager(object):

    def test_save_new(self, fact_manager, fact):
        """Make sure instance is saved and returned."""
        dbus_fact = helpers.hamster_to_dbus_fact(fact)
        result = fact_manager.Save(dbus_fact)
        result = helpers.dbus_to_hamster_fact(result)
        assert fact.as_tuple(include_pk=False) == result.as_tuple(include_pk=False)

    def test_remove(self, fact_manager, stored_fact):
        """Make sure instance is removed."""
        result = fact_manager.Remove(stored_fact.pk)
        assert result is None
        # [FIXME]
        # with pytest.raises(KeyError):
        #     store.facts.get(pk)

    def test_get(self, fact_manager, stored_fact):
        """Make sure instance is returned."""
        result = fact_manager.Get(stored_fact.pk)
        result = helpers.dbus_to_hamster_fact(result)
        assert result == stored_fact

    # [FIXME]
    # This should be expanded
    def test_get_all(self, fact_manager, stored_fact_batch_factory):
        """Make sure we get all matching instances."""
        stored_fact_batch_factory(5)
        result = fact_manager.GetAll('', '', '')
        assert len(result) == 5



# -*- coding: utf-8 -*-

"""
Unittests for ``hamster_dbus.storage.CategoryManager``.

Please refer to ``__init__.py`` for general details.
"""

from __future__ import absolute_import, unicode_literals

import subprocess

import dbus
import dbusmock
from hamster_lib import objects as lib_objects

from hamster_dbus import storage

from . import common
from .. import factories


class BaseTestCategoryManager(common.HamsterDBusManagerTestCase):
    """Base test case that provides infrastructure common to all other test cases."""

    def setUp(self):
        """
        Setup a mock ``CategoryManager`` object and provide a convenient interface.

        By adding methods to ``self.interface`` individual tests can mock an available
        dbus object while ``self.manager`` provides easy access to an actual
        ``CategoryManager`` instance. By explicitly passing our test session bus
        we make sure the manager queries against the right bus/object.
        """
        self.service_mock = self.spawn_server(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/CategoryManager',
            'org.projecthamster.HamsterDBus.CategoryManager1',
            stdout=subprocess.PIPE
        )

        self.dbus_object = self.dbus_con.get_object(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/CategoryManager'
        )

        self.interface = dbus.Interface(self.dbus_object, dbusmock.MOCK_IFACE)
        self.manager = storage.CategoryManager(bus=self.dbus_con)


class TestSave(BaseTestCategoryManager):

    def setUp(self):
        """Test setup."""
        super(TestSave, self).setUp()
        self.existing_category = factories.CategoryFactory(pk=1)

    def test_save(self):
        """Make sure a ``Category`` instance is returned."""
        self.dbus_object.AddMethod('', 'Save', '(is)', '(is)', 'ret = (1, "foo")')

        result = self.manager.save(self.existing_category)
        self.assertIsInstance(result, lib_objects.Category)

    def test_save_non_category(self):
        """Make sure that passing anything but a ``Category`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.save('foobar')


class TestGetOrCreate(BaseTestCategoryManager):

    def setUp(self):
        """Test setup."""
        super(TestGetOrCreate, self).setUp()
        self.new_category = factories.CategoryFactory()
        self.existing_category = factories.CategoryFactory(pk=1)

    def test_get_or_create_get(self):
        """Make sure a ``Category`` instance is returned."""
        self.dbus_object.AddMethod('', 'GetOrCreate', '(is)', '(is)', 'ret = (1, "foo")')

        result = self.manager.get_or_create(self.existing_category)
        self.assertIsInstance(result, lib_objects.Category)

    def test_get_or_create_create(self):
        """Make sure a ``Category`` instance is returned."""
        self.dbus_object.AddMethod('', 'GetOrCreate', '(is)', '(is)', 'ret = (1, "foo")')

        result = self.manager.get_or_create(self.new_category)
        self.assertIsInstance(result, lib_objects.Category)

    def test_non_category_instance(self):
        """Make sure that passing anything but a ``Category`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_or_create('foobar')


class TestRemove(BaseTestCategoryManager):

    def setUp(self):
        """Test setup."""
        super(TestRemove, self).setUp()
        self.existing_category = factories.CategoryFactory(pk=1)

    def test_remove(self):
        """Make sure a ``None`` is returned."""
        self.dbus_object.AddMethod('', 'Remove', 'i', '', '')
        result = self.manager.remove(self.existing_category)
        self.assertIsNone(result)

    def test_non_category_instance(self):
        """Make sure that passing anything but a ``Category`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.remove(1)


class TestGet(BaseTestCategoryManager):

    def setUp(self):
        """Test setup."""
        super(TestGet, self).setUp()
        self.existing_category = factories.CategoryFactory(pk=1)

    def test_get(self):
        """Make sure a ``Category`` instance is returned."""
        self.dbus_object.AddMethod('', 'Get', 'i', '(is)', 'ret = (1, "foo")')
        result = self.manager.get(self.existing_category.pk)
        self.assertIsInstance(result, lib_objects.Category)


class TestGetByName(BaseTestCategoryManager):
    def test_get_by_name(self):
        """Make sure a ``Category`` instance is returned."""
        self.dbus_object.AddMethod('', 'GetByName', 's', '(is)', 'ret = (1, "foo")')
        # [FIXME]
        # Due to problems with dbusmock under python 2
        result = self.manager.get_by_name('foobar')
        self.assertIsInstance(result, lib_objects.Category)


class TestGetAll(BaseTestCategoryManager):
    def test_get_all(self):
        """Make sure an iterator of ``Category`` instances is returned."""
        self.dbus_object.AddMethod('', 'GetAll', '', 'a(is)', 'ret = [(1, "foo"), (2, "bar")]')
        result = self.manager.get_all()
        for each in result:
            self.assertIsInstance(each, lib_objects.Category)



"""
Unittests for the dbus-storage backend.

At first glance it may seem strange that we add mocked service methods not as
part of the test setup but in each individual test. This is due to
``dbus-python``s strange way to handle mocking return values. Whilst all tests
related to a particular test class deal with the same method, they may expect
different return values specific to their test goals. As ``AddMethod`` does not
really seem to allow for dynamic return values it seems cleaner and more
transparent to provide them with each individual test explicitly.

Please not that the unit tests included here do *mock* the underlying dbus
service.  This mean in particular that they make assumptions about the services
signature.  As a consequence those tests will not be able to make sure that the
tested storage backend will actually work against a given service implementation
(such as provided by ``hamster_dbus.objects``). For this to be tested, dedicated
integration tests are needed.

Most manager methods really only call the relevant de/serialization functions
before/after the respective dbus object method call. As the method call itself
is mocked, and the de/serialization functions are tested separately there is
relatively little to test here for now and we mainly make sure that the returned
types match our expectation. Still, even those basic test coverage will provide
useful guards against regression bugs as they ensure the relevant manager
methods are called at least once.
"""



# -*- coding: utf-8 -*-

"""Module to provide a common base ``TestCase``."""


from __future__ import absolute_import, unicode_literals

import dbusmock


class HamsterDBusManagerTestCase(dbusmock.DBusTestCase):
    """
    Common testcase for storage backend unittests.

    This test case makes sure tests are run against a new private session bus
    instance and provides easy access to the underlying dbus connection.
    """

    @classmethod
    def setUpClass(cls):
        """Setup new private session bus."""
        cls.start_session_bus()
        cls.dbus_con = cls.get_dbus()

    def tearDown(self):
        """Terminate any service launched by the test case."""
        self.service_mock.terminate()
        self.service_mock.wait()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2017 Eric Goller <eric.goller@ninjaduck.solutions>

# This file is part of 'hamster-dbus'.
#
# 'hamster-dbus' is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# 'hamster-dbus' is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  'hamster-dbus'.  If not, see <http://www.gnu.org/licenses/>.

"""
This modules provides dbus backend conforming to the ``hamster_lib.storage`` API.

Using this module allows clients direct all of their storage backend calls against
a corresponding dbus service.
"""

# Whilst it usually preferred to inherit from ``hamster_lib.storage`` and just
# implement the specific functional backend methods such an approach would have
# meant that the mirroring dbus-service would have to export quite a few
# 'private' manager methods.  For this reason we opted against it and instead
# implement the relevant public manager methods 'from scratch'. The price for
# that is that we duplicate errorhandling/sanity check code.


from __future__ import absolute_import, unicode_literals

import datetime
from gettext import gettext as _

import dbus
import hamster_lib.objects as lib_objects
import hamster_lib.storage as lib_storage
from future.utils import python_2_unicode_compatible
from six import text_type

import hamster_dbus.helpers as helpers


@python_2_unicode_compatible
class DBusStore(lib_storage.BaseStore):
    """Store class for hamster-dbus storage backend."""

    def __init__(self, config, bus=None):
        """
        Initialize a new instance.

        Args:
            config (dict): Dictionary containing config data.
            bus (dbus.bus.BusConnection, optional): Connection to be used when
            querying dbus objects. If ``None``, ``dbus.SessionBus()`` will be
            used.

        Returns:
            DBusStore: DBusStore instance.
        """
        if bus is None:
            bus = dbus.SessionBus()
        self._bus = bus
        self.config = config
        self.categories = CategoryManager(self._bus)
        self.activities = ActivityManager(self._bus)
        self.tags = TagManager(self._bus)
        self.facts = FactManager(self._bus)

    def cleanup(self):
        """Teardown chores."""
        return None


@python_2_unicode_compatible
class CategoryManager(object):
    """Class to handle categories."""

    def __init__(self, bus):
        """
        Instantiate class.

        Args:
            bus (dbus.bus.BusConnection): Connection to query against.
        """
        bus_name = 'org.projecthamster.HamsterDBus'
        object_path = '/org/projecthamster/HamsterDBus/CategoryManager'
        interface_name = 'org.projecthamster.HamsterDBus.CategoryManager1'
        dbus_object = bus.get_object(bus_name, object_path)
        self._interface = dbus.Interface(dbus_object, interface_name)

    def save(self, category):
        """
        Save a Category.

        Args:
            category (hamster_lib.Category): Category instance to be saved.

        Returns:
            hamster_lib.Category: Saved Category

        Raises:
            TypeError: If the ``category`` parameter is not a valid ``Category`` instance.
        """
        if not isinstance(category, lib_objects.Category):
            message = _("You need to pass a hamster category")
            raise TypeError(message)

        dbus_category = helpers.hamster_to_dbus_category(category)
        result = self._interface.Save(dbus_category)
        return helpers.dbus_to_hamster_category(result)

    def get_or_create(self, category):
        """
        Check if we already got a category with that name, if not create one.

        For details see the corresponding method in ``hamster_lib.storage``.

        Args:
            category (hamster_lib.Category or None): The categories.

        Returns:
            hamster_lib.Category or None: The retrieved or created category. Either way,
                the returned Category will contain all data from the backend, including
                its primary key.

        Raises:
            TypeError: If ``category`` is not a ``lib_objects.Category`` instance.
        """
        if not isinstance(category, lib_objects.Category):
            message = _("You need to pass a hamster category")
            raise TypeError(message)

        dbus_category = helpers.hamster_to_dbus_category(category)
        result = self._interface.GetOrCreate(dbus_category)
        return helpers.dbus_to_hamster_category(result)

    def remove(self, category):
        """
        Remove a category.

        Any ``Activity`` referencing the passed category will be set to
        ``Activity().category=None``.

        Args:
            category (hamster_lib.Category): Category to be updated.

        Returns:
            None: If everything went ok.

        Raises:
            TypeError: If category passed is not an hamster_lib.Category instance.
        """
        if not isinstance(category, lib_objects.Category):
            message = _("You need to pass a hamster category")
            raise TypeError(message)

        self._interface.Remove(category.pk)
        return None

    def get(self, pk):
        """
        Get an ``Category`` by its primary key.

        Args:
            pk (int): Primary key of the ``Category`` to be fetched.

        Returns:
            hamster_lib.Category: ``Category`` with given primary key.
        """
        result = self._interface.Get(int(pk))
        return helpers.dbus_to_hamster_category(result)

    def get_by_name(self, name):
        """
        Look up a category by its name.

        Args:
            name (str): Unique name of the ``Category`` to we want to fetch.

        Returns:
            hamster_lib.Category: ``Category`` with given name.
        """
        result = self._interface.GetByName(name)
        return helpers.dbus_to_hamster_category(result)

    def get_all(self):
        """
        Return a list of all categories.

        Returns:
            list: List of ``Categories``, ordered by ``lower(name)``.
        """
        result = self._interface.GetAll()
        return [helpers.dbus_to_hamster_category(category) for category in result]


@python_2_unicode_compatible
class ActivityManager(object):
    """Class to handle activities."""

    def __init__(self, bus):
        """
        Instantiate class.

        Args:
            bus (dbus.bus.BusConnection): Connection to query against.
        """
        bus_name = 'org.projecthamster.HamsterDBus'
        object_path = '/org/projecthamster/HamsterDBus/ActivityManager'
        interface_name = 'org.projecthamster.HamsterDBus.ActivityManager1'
        dbus_object = bus.get_object(bus_name, object_path)
        self._interface = dbus.Interface(dbus_object, interface_name)

    def save(self, activity):
        """
        Save an Activivty.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not isinstance(activity, lib_objects.Activity):
            message = _("You need to pass a ``hamster_lib.objects.Activity`` instance")
            raise TypeError(message)

        dbus_activity = helpers.hamster_to_dbus_activity(activity)
        result = self._interface.Save(dbus_activity)
        return helpers.dbus_to_hamster_activity(result)

    def get_or_create(self, activity):
        """
        Convenience method to either get an activity matching the specs or create a new one.

        For details see the corresponding method in ``hamster_lib.storage``.

        Args:
            activity (hamster_lib.Activity): The activity we want.

        Returns:
            hamster_lib.Activity: The retrieved or created activity

        Raises:
            TypeError: If ``activity`` is not a ``lib_objects.Activity`` instance.
        """
        if not isinstance(activity, lib_objects.Activity):
            message = _("You need to pass a ``hamster_lib.objects.Activity`` instance")
            raise TypeError(message)

        dbus_activity = helpers.hamster_to_dbus_activity(activity)
        result = self._interface.GetOrCreate(dbus_activity)
        return helpers.dbus_to_hamster_activity(result)

    def remove(self, activity):
        """
        Remove an ``Activity`` from the database.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not isinstance(activity, lib_objects.Activity):
            message = _("You need to pass a ``hamster_lib.objects.Activity`` instance")
            raise TypeError(message)

        dbus_activity = helpers.hamster_to_dbus_activity(activity)
        self._interface.Remove(dbus_activity.pk)
        return True

    def get(self, pk):
        """
        Return an activity based on its primary key.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        result = self._interface.Get(pk)
        return helpers.dbus_to_hamster_activity(result)

    def get_by_composite(self, name, category):
        """
        Lookup for unique 'name/category.name'-composite key.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not (isinstance(category, lib_objects.Category) or (category is None)):
            message = _("You need to pass a hamster_lib.objects.Category instance or None")
            raise TypeError(message)

        dbus_category = helpers.hamster_to_dbus_category(category)
        name = text_type(name)
        result = self._interface.GetByComposite(name, dbus_category)
        return helpers.dbus_to_hamster_activity(result)

    def get_all(self, category=False, search_term=''):
        """
        Return all matching activities.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if isinstance(category, lib_objects.Category):
            category = category.pk
        elif category is None:
            category = -1
        elif category is False:
            category = -2
        else:
            message = _(
                "'category' needs to be either a 'hamster_lib.objects.Category' instance,"
                " 'False' or 'None'."
            )
            raise TypeError(message)

        search_term = text_type(search_term)

        result = self._interface.GetAll(category, search_term)
        return [helpers.dbus_to_hamster_activity(activity) for activity in result]


@python_2_unicode_compatible
class TagManager(object):
    """Class to handle tags."""

    def __init__(self, bus):
        """
        Instantiate class.

        Args:
            bus (dbus.bus.BusConnection): Connection to query against.
        """
        bus_name = 'org.projecthamster.HamsterDBus'
        object_path = '/org/projecthamster/HamsterDBus/TagManager'
        interface_name = 'org.projecthamster.HamsterDBus.TagManager1'
        dbus_object = bus.get_object(bus_name, object_path)
        self._interface = dbus.Interface(dbus_object, interface_name)

    def save(self, tag):
        """
        Save a tag.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not isinstance(tag, lib_objects.Tag):
            message = _("You need to pass a ``hamster_lib.objects.Tag`` instance")
            raise TypeError(message)

        dbus_tag = helpers.hamster_to_dbus_tag(tag)
        result = self._interface.Save(dbus_tag)
        return helpers.dbus_to_hamster_tag(result)

    def get_or_create(self, tag):
        """
        Check if we already got a tag with that name, if not create one.

        For details see the corresponding method in ``hamster_lib.storage``.

        Args:
            tag (hamster_lib.Tag): The tag we want.

        Returns:
            hamster_lib.Tag: The retrieved or created tag.

        Raises:
            TypeError: If ``tag`` is not a ``lib_objects.Tag`` instance.
        """
        if not isinstance(tag, lib_objects.Tag):
            message = _("You need to pass a ``hamster_lib.objects.Tag`` instance")
            raise TypeError(message)

        dbus_tag = helpers.hamster_to_dbus_tag(tag)
        result = self._interface.GetOrCreate(dbus_tag)
        return helpers.dbus_to_hamster_tag(result)

    def remove(self, tag):
        """
        Remove a tag.

        Any ``Fact`` referencing the passed tag will have this tag removed.

        Args:
            tag (hamster_lib.Tag): Tag to be updated.

        Returns:
            None: If everything went ok.

        Raises:
            TypeError: If tag passed is not an hamster_lib.Tag instance.
        """
        if not isinstance(tag, lib_objects.Tag):
            message = _("You need to pass a hamster tag")
            raise TypeError(message)

        self._interface.Remove(tag.pk)
        return None

    def get(self, pk):
        """
        Get an ``Tag`` by its primary key.

        Args:
            pk (int): Primary key of the ``Tag`` to be fetched.

        Returns:
            hamster_lib.Tag: ``Tag`` with given primary key.
        """
        result = self._interface.Get(int(pk))
        return helpers.dbus_to_hamster_tag(result)

    def get_by_name(self, name):
        """
        Look up a tag by its name.

        Args:
            name (text_type): Unique name of the ``Tag`` to we want to fetch.

        Returns:
            hamster_lib.Tag: ``Tag`` with given name.
        """
        result = self._interface.GetByName(name)
        return helpers.dbus_to_hamster_tag(result)

    def get_all(self):
        """
        Return a list of all tags.

        Returns:
            list: List of ``Tags``, ordered by ``lower(name)``.
        """
        result = self._interface.GetAll()
        return [helpers.dbus_to_hamster_tag(tag) for tag in result]


@python_2_unicode_compatible
class FactManager(object):
    """Class to handle facts."""

    def __init__(self, bus):
        """
        Instantiate class.

        Args:
            bus (dbus.bus.BusConnection): Connection to query against.
        """
        bus_name = 'org.projecthamster.HamsterDBus'
        object_path = '/org/projecthamster/HamsterDBus/FactManager'
        interface_name = 'org.projecthamster.HamsterDBus.FactManager1'
        dbus_object = bus.get_object(bus_name, object_path)
        self._interface = dbus.Interface(dbus_object, interface_name)

    def save(self, fact):
        """
        Save a Fact.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not isinstance(fact, lib_objects.Fact):
            message = _("You need to pass a ``hamster_lib.objects.Fact`` instance")
            raise TypeError(message)

        dbus_fact = helpers.hamster_to_dbus_fact(fact)
        result = self._interface.Save(dbus_fact)
        return helpers.dbus_to_hamster_fact(result)

    def remove(self, fact):
        """
        Remove a Fact.

        For details see the corresponding method in ``hamster_lib.storage``.
        """
        if not isinstance(fact, lib_objects.Fact):
            message = _("You need to pass a ``hamster_lib.objects.Fact`` instance")
            raise TypeError(message)

        self._interface.Remove(fact.pk)

    def get(self, pk):
        """
        Return a ``Fact`` by its primary key.

        Args:
            pk (int): Primary key of the ``Fact to be retrieved``.

        Returns:
            hamster_lib.Fact: The ``Fact`` corresponding to the primary key.
        """
        result = self._interface.Get(int(pk))
        return helpers.dbus_to_hamster_fact(result)

    def get_all(self, start=None, end=None, filter_term=''):
        """
        Return all facts within a given timeframe.

        The 'timeframe' begins at the begining of ``start`` and
        concludes at the end of ``end``.

        Args:
            start (datetime.datetime, datetime.date, datetime.time or None, optional): Consider
                only Facts starting at or after this date. Alternatively you can
                also pass a ``datetime.datetime`` object in which case its own
                time will be considered instead of the default ``day_start``
                or a ``datetime.time`` which will be considered as today.
                Defaults to ``None``.
            end (datetime.datetime, datetime.date, datetime.time or None, optional): Consider
                only Facts ending before or at this date. Alternatively you can
                also pass a ``datetime.datetime`` object in which case its own
                time will be considered instead of the default ``day_start``
                or a ``datetime.time`` which will be considered as today.
                Defaults to ``None``.
            filter_term (str, optional): Only consider ``Facts`` with this
                string as part of their associated ``Activity.name``

        Returns:
            list: List of ``Fact``s matching given specifications.

        Raises:
            TypeError: If ``start`` or ``end`` are not ``datetime.date``, ``datetime.time`` or
                ``datetime.datetime`` objects.
            ValueError: If ``end`` is before ``start``.

        Note:
            * This public function only provides some sanity checks and normalization. The actual
                backend query is handled by ``_get_all``.
            * ``search_term`` should be prefixable with ``not`` in order to invert matching.
            * This does only return proper facts and does not include any existing 'ongoing fact'.
        """
        if not (isinstance(start, datetime.datetime) or isinstance(start, datetime.date) or (
                isinstance(start, datetime.time) or (start is None))):
            raise TypeError
        if not (isinstance(end, datetime.datetime) or isinstance(end, datetime.date) or (
                isinstance(end, datetime.time) or (end is None))):
            raise TypeError

        if start and end and (end <= start):
            message = _("End value can not be earlier than start!")
            raise ValueError(message)

        start = helpers.datetime_to_text(start)
        end = helpers.datetime_to_text(end)
        filter_term = text_type(filter_term)
        result = self._interface.GetAll(start, end, filter_term)
        return [helpers.dbus_to_hamster_fact(fact) for fact in result]

    def get_today(self):
        """
        Return all facts for today, while respecting ``day_start``.

        Returns:
            list: List of ``Fact`` instances.

        Note:
            * This does only return proper facts and does not include any existing 'ongoing fact'.
        """
        result = self._interface.GetToday()
        return [helpers.dbus_to_hamster_fact(fact) for fact in result]

    def stop_tmp_fact(self):
        """
        Stop current 'ongoing fact'.

        Returns:
            hamster_lib.Fact: The stored fact.

        Raises:
            ValueError: If there is no currently 'ongoing fact' present.
        """
        result = self._interface.StopTmpFact()
        return helpers.dbus_to_hamster_fact(result)

    def cancel_tmp_fact(self):
        """
        Provide a way to stop an 'ongoing fact' without saving it in the backend.

        Returns:
            None: If everything worked as expected.

        Raises:
            KeyError: If no ongoing fact is present.
        """
        self._interface.CancelTmpFact()
        return None

    def get_tmp_fact(self):
        """
        Provide a way to retrieve any existing 'ongoing fact'.

        Returns:
            hamster_lib.Fact: An instance representing our current 'ongoing fact'.capitalize

        Raises:
            KeyError: If no ongoing fact is present.
        """
        result = self._interface.GetTmpFact()
        return helpers.dbus_to_hamster_fact(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Packing metadata for setuptools."""


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'future',
    'hamster_lib',
]

setup(
    name='hamster-dbus',
    version='0.10.0',
    description="A dbus interface to hamster-lib.",
    long_description=readme + '\n\n' + history,
    author="Eric Goller",
    author_email='eric.goller@ninjaduck.solutions',
    url='https://github.com/elbenfreund/hamster-dbus',
    packages=[
        'hamster_dbus',
    ],
    package_dir={'hamster_dbus':
                 'hamster_dbus'},
    package_data={'hamster-dbus': ['examples/*']},
    install_requires=requirements,
    entry_points={
        'console_scripts': ['hamster-dbus-service = hamster_dbus.hamster_dbus_service:_main']
    },
    license="GPL3",
    zip_safe=False,
    keywords='hamster-dbus',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
)



"""General fixtures."""
from __future__ import absolute_import, unicode_literals

import datetime
import os
import signal
import subprocess
import time

import dbus
import dbusmock
import fauxfactory
import pytest
from pytest_factoryboy import register

from hamster_dbus import helpers

from .. import factories

register(factories.CategoryFactory)
register(factories.ActivityFactory)
register(factories.TagFactory)
register(factories.FactFactory)

# [FIXME]
# At several places we use ``faker.word`` instead of the original faked value.
# If I remember correctly this is due to probems with ``dbusmock`` under python2
# rather than our own code, but still this should at least be documented
# somewhere.


@pytest.fixture
def private_session_bus(request):
    """
    Launch a new 'private' session bus and return the connection instance.

    This is done so that we do not 'pollute' the testing users session bus.

    We can not use dbusmock classmethod as it does not return the PID, which
    we need in order to kill the process in the finalizer.
    """
    def fin():
        dbusmock.testcase.DBusTestCase.stop_dbus(pid)

    def start_bus():
        argv = ['dbus-launch']
        out = subprocess.check_output(argv, universal_newlines=True)
        variables = {}
        for line in out.splitlines():
            (k, v) = line.split('=', 1)
            variables[k] = v
        return (int(variables['DBUS_SESSION_BUS_PID']),
                variables['DBUS_SESSION_BUS_ADDRESS'])

    request.addfinalizer(fin)

    pid, address = start_bus()
    os.environ['DBUS_SESSION_BUS_ADDRESS'] = address
    time.sleep(0.5)

    dbus_con = dbus.bus.BusConnection(os.environ['DBUS_SESSION_BUS_ADDRESS'])
    return dbus_con


@pytest.fixture
def live_service(request, private_session_bus):
    """
    Provide a running hamster service hooked into a private session bus.

    Returns the 'daemon' ``Popen`` object as well as the bus connection it has
    been launched on.

    Note: The way a launched service determines the bus to connect to is by
    inspecting ``DBUS_SESSION_BUS_ADDRESS`` ENVVAR. If this would be empty,
    the default session bus is used.
    """
    def fin():
        os.kill(daemon.pid, signal.SIGTERM)

    request.addfinalizer(fin)

    daemon = subprocess.Popen(['hamster_dbus/hamster_dbus_service.py', 'server'])
    dbusmock.testcase.DBusTestCase.wait_for_bus_object(
        'org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus/ActivityManager',
    )
    bus = private_session_bus
    return daemon, bus


@pytest.fixture
def hamster_dbus_interface(request, live_service):
    """Provide a convenient object hook to our hamster-dbus service."""
    daemon, bus = live_service
    object_ = bus.get_object('org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus')
    interface = dbus.Interface(object_,
        dbus_interface='org.projecthamster.HamsterDBus1')
    return interface


@pytest.fixture
def category_manager(request, live_service):
    """Provide a convenient object hook to our hamster-dbus service."""
    daemon, bus = live_service
    object_ = bus.get_object('org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus/CategoryManager')
    interface = dbus.Interface(object_,
        dbus_interface='org.projecthamster.HamsterDBus.CategoryManager1')
    return interface


@pytest.fixture
def activity_manager(request, live_service):
    """Provide a convenient object hook to our hamster-dbus service."""
    daemon, bus = live_service
    object_ = bus.get_object('org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus/ActivityManager')
    interface = dbus.Interface(object_,
        dbus_interface='org.projecthamster.HamsterDBus.ActivityManager1')
    return interface


@pytest.fixture
def tag_manager(request, live_service):
    """Provide a convenient object hook to our hamster-dbus service."""
    daemon, bus = live_service
    object_ = bus.get_object('org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus/TagManager')
    interface = dbus.Interface(object_,
        dbus_interface='org.projecthamster.HamsterDBus.TagManager1')
    return interface


@pytest.fixture
def fact_manager(request, live_service):
    """Provide a convenient object hook to our hamster-dbus service."""
    daemon, bus = live_service
    object_ = bus.get_object('org.projecthamster.HamsterDBus',
        '/org/projecthamster/HamsterDBus/FactManager')
    interface = dbus.Interface(object_,
        dbus_interface='org.projecthamster.HamsterDBus.FactManager1')
    return interface


# Data
@pytest.fixture(params=[
    fauxfactory.gen_alpha(),
    fauxfactory.gen_utf8(),
    fauxfactory.gen_latin1(),
    fauxfactory.gen_cjk(),
])
def category_name_parametrized(request):
    """Provide a huge variety of possible ``Category.name`` strings."""
    return request.param


@pytest.fixture(params=[
    fauxfactory.gen_alpha(),
    fauxfactory.gen_utf8(),
    fauxfactory.gen_latin1(),
    fauxfactory.gen_cjk(),
])
def activity_name_parametrized(request):
    """Provide a huge variety of possible ``Activity.name`` strings."""
    return request.param


@pytest.fixture(params=[
    fauxfactory.gen_alpha(),
    fauxfactory.gen_utf8(),
    fauxfactory.gen_latin1(),
    fauxfactory.gen_cjk(),
])
def tag_name_parametrized(request):
    """Provide a huge variety of possible ``Tag.name`` strings."""
    return request.param


# Stored instances
@pytest.fixture
def stored_category_factory(request, category_manager, category_factory, faker):
    """
    A factory for category instances that are present in our persistent store.

    Because we do not have access to the actual store we need to assume that
    the ``Save`` method works as expected.
    """
    def factory(**kwargs):
        category = category_factory.build(**kwargs)
        result = category_manager.Save(helpers.hamster_to_dbus_category(category))
        return helpers.dbus_to_hamster_category(result)
    return factory


@pytest.fixture
def stored_category(request, stored_category_factory):
    """A singe persistent category instances."""
    return stored_category_factory()


@pytest.fixture
def stored_category_batch_factory(request, stored_category_factory, faker):
    """A factory for category instances that are present in our persistent store."""
    def factory(amount):
        categories = []
        for i in range(amount):
            categories.append(stored_category_factory(name=faker.word()))
        return categories
    return factory


@pytest.fixture
def stored_activity_factory(request, activity_manager, activity_factory, faker):
    """
    A factory for activity instances that are present in our persistent store.

    Because we do not have access to the actual store we need to assume that
    the ``Save`` method works as expected.
    """
    def factory(**kwargs):
        activity = activity_factory.build(**kwargs)
        result = activity_manager.Save(helpers.hamster_to_dbus_activity(activity))
        return helpers.dbus_to_hamster_activity(result)
    return factory


@pytest.fixture
def stored_activity(request, stored_activity_factory):
    """A singe persistent activity instances."""
    return stored_activity_factory()


@pytest.fixture
def stored_activity_batch_factory(request, stored_activity_factory, faker):
    """Factory for batch creating persistent activity instances."""
    def factory(amount):
        activities = []
        for i in range(amount):
            activities.append(stored_activity_factory(name=faker.word()))
        return activities
    return factory


@pytest.fixture
def stored_tag_factory(request, tag_manager, tag_factory, faker):
    """
    A factory for tag instances that are present in our persistent store.

    Because we do not have access to the actual store we need to assume that
    the ``Save`` method works as expected.
    """
    def factory(**kwargs):
        tag = tag_factory.build(**kwargs)
        result = tag_manager.Save(helpers.hamster_to_dbus_tag(tag))
        return helpers.dbus_to_hamster_tag(result)
    return factory


@pytest.fixture
def stored_tag(request, stored_tag_factory):
    """A singe persistent tag instances."""
    return stored_tag_factory()


@pytest.fixture
def stored_tag_batch_factory(request, stored_tag_factory, faker):
    """A factory for tag instances that are present in our persistent store."""
    def factory(amount):
        tags = []
        for i in range(amount):
            tags.append(stored_tag_factory(name=faker.word()))
        return tags
    return factory


@pytest.fixture
def stored_fact_factory(request, fact_manager, fact_factory, faker):
    """A factory for fact instances that are present in our persistent store."""
    def factory(**kwargs):
        fact = fact_factory.build(**kwargs)
        result = fact_manager.Save(helpers.hamster_to_dbus_fact(fact))
        return helpers.dbus_to_hamster_fact(result)
    return factory


@pytest.fixture
def stored_fact(request, stored_fact_factory):
    """A singe persistent fact instances."""
    return stored_fact_factory()


@pytest.fixture
def stored_fact_batch_factory(request, stored_fact_factory, faker):
    """
    Factory for batch creating persistent fact instances.

    Because we do not have access to the actual store we need to assume that
    the ``Save`` method works as expected.
    """
    def factory(amount):
        facts = []
        old_start = datetime.datetime.now()
        offset = datetime.timedelta(hours=4)
        for i in range(amount):
            start = old_start + offset
            fact = stored_fact_factory(start=start)
            facts.append(fact)
            old_start = start
        return facts
    return factory



# -*- coding: utf-8 -*-

"""
Unittests for ``hamster_dbus.storage.ActivityManager``.

Please refer to ``__init__.py`` for general details.
"""

from __future__ import absolute_import, unicode_literals

import subprocess

import dbus
import dbusmock
from hamster_lib import objects as lib_objects

from hamster_dbus import storage

from . import common
from .. import factories


class BaseTestActivityManager(common.HamsterDBusManagerTestCase):
    """Base test case that provides infrastructure common to all other test cases."""

    def setUp(self):
        """
        Setup a mock ``ActivityManager`` object and provide a convenient interface.

        By adding methods to ``self.interface`` individual tests can mock an available
        dbus object while ``self.manager`` provides easy access to an actual
        ``ActivityManager`` instance. By explicitly passing our test session bus
        we make sure the manager queries against the right bus/object.
        """
        self.service_mock = self.spawn_server(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/ActivityManager',
            'org.projecthamster.HamsterDBus.ActivityManager1',
            stdout=subprocess.PIPE
        )

        self.dbus_object = self.dbus_con.get_object(
            'org.projecthamster.HamsterDBus',
            '/org/projecthamster/HamsterDBus/ActivityManager'
        )

        self.interface = dbus.Interface(self.dbus_object, dbusmock.MOCK_IFACE)
        self.manager = storage.ActivityManager(bus=self.dbus_con)


class TestSave(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestSave, self).setUp()
        self.existing_activity = factories.ActivityFactory(pk=1)

    def test_existing(self):
        """Make sure saving an existing activity returns an ``Activity`` instance."""
        self.dbus_object.AddMethod(
            '', 'Save', '(is(is)b)', '(is(is)b)', 'ret = (1, "foo", (1, "bar"), False)'
        )

        result = self.manager.save(self.existing_activity)
        self.assertIsInstance(result, lib_objects.Activity)

    def test_save_non_activivty(self):
        """Make sure that saving anything but an ``Activity`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.save('foobar')


class TestGetOrCreate(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestGetOrCreate, self).setUp()
        self.new_activivty = factories.ActivityFactory(pk=None)
        self.existing_activity = factories.ActivityFactory(pk=1)

    def test_existing_activity(self):
        """Make sure passing an existing activity returns an ``Activity`` instance."""
        self.dbus_object.AddMethod(
            '', 'GetOrCreate', '(is(is)b)', '(is(is)b)', 'ret = (1, "foo", (1, "bar"), False)'
        )

        result = self.manager.get_or_create(self.existing_activity)
        self.assertIsInstance(result, lib_objects.Activity)

    def test_new_activity(self):
        """Make sure passing a new activity returns an ``Activity`` instance."""
        self.dbus_object.AddMethod(
            '', 'GetOrCreate', '(is(is)b)', '(is(is)b)', 'ret = (1, "foo", (1, "bar"), False)'
        )

        result = self.manager.get_or_create(self.new_activivty)
        self.assertIsInstance(result, lib_objects.Activity)

    def test_save_non_activivty(self):
        """Make sure that saving anything but an ``Activity`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_or_create('string')


class TestRemove(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestRemove, self).setUp()
        self.existing_activity = factories.ActivityFactory(pk=1)

    def test_remove_existing(self):
        """Make sure that removing an ``Activity`` returns ``True``."""
        self.dbus_object.AddMethod('', 'Remove', 'i', '', '')

        result = self.manager.remove(self.existing_activity)
        self.assertTrue(result)

    def test_save_non_activivty(self):
        """Make sure that saving anything but an ``Activity`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.remove('string')


class TestGet(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestGet, self).setUp()
        self.existing_activity = factories.ActivityFactory(pk=1)

    def test_get_existing(self):
        """Make sure an ``Activity`` instance is returned."""

        self.dbus_object.AddMethod(
            '', 'Get', 'i', '(is(is)b)', 'ret = (1, "foo", (1, "bar"), False)'
        )

        result = self.manager.get(self.existing_activity.pk)
        self.assertIsInstance(result, lib_objects.Activity)


class TestGetByComposite(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestGetByComposite, self).setUp()
        self.existing_activity = factories.ActivityFactory(pk=1)

    def test_with_category(self):
        """Make sure an ``Activity`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'GetByComposite', 's(is)', '(is(is)b)', 'ret = (1, "foo", (1, "bar"), False)'
        )

        result = self.manager.get_by_composite(
            # [FIXME]
            # It looks like ``python-dbusmock`` is not truely python 2
            # compatible and uses problematic ``str``-casts.
            # self.existing_activity.name, self.existing_activity.category
            'fooo', self.existing_activity.category
        )
        self.assertIsInstance(result, lib_objects.Activity)

    def test_without_category(self):
        """Make sure an ``Activity`` instance is returned."""
        self.dbus_object.AddMethod(
            '', 'GetByComposite', 's(is)', '(is(is)b)', 'ret = (1, "foo", (-2, ""), False)'
        )

        # [FIXME]
        # It looks like ``python-dbusmock`` is not truely python 2
        # compatible and uses problematic ``str``-casts.
        # self.existing_activity.name, self.existing_activity.category
        result = self.manager.get_by_composite('fooo', None)
        self.assertIsInstance(result, lib_objects.Activity)

    def test_invalid_category_type(self):
        """Make sure that anything but an ``Activity`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_by_composite('foo', 'category1')


class TestGetAll(BaseTestActivityManager):

    def setUp(self):
        """Test setup."""
        super(TestGetAll, self).setUp()
        self.existing_category = factories.CategoryFactory(pk=1)

    def test_filter_by_existing_category(self):
        """Make sure an iterator of ``Activity`` instances is returned."""
        self.dbus_object.AddMethod(
            '', 'GetAll', 'is', 'a(is(is)b)', 'ret = [(1, "foo", (1, "bar"), False)]'
        )

        result = self.manager.get_all(self.existing_category, 'foobar')
        for each in result:
            self.assertIsInstance(each, lib_objects.Activity)

    def test_filter_without_category(self):
        """Make sure an iterator of ``Activity`` instances is returned."""
        self.dbus_object.AddMethod(
            '', 'GetAll', 'is', 'a(is(is)b)', 'ret = [(1, "foo", (1, "bar"), False)]'
        )

        result = self.manager.get_all(None, 'foobar')
        for each in result:
            self.assertIsInstance(each, lib_objects.Activity)

    def test_allow_all_categories(self):
        """Make sure an iterator of ``Activity`` instances is returned."""
        self.dbus_object.AddMethod(
            '', 'GetAll', 'is', 'a(is(is)b)', 'ret = [(1, "foo", (1, "bar"), False)]'
        )

        result = self.manager.get_all(category=False, search_term='foo')
        for each in result:
            self.assertIsInstance(each, lib_objects.Activity)

    def test_invalid_category(self):
        """Make sure that anything but an ``Category`` instance throws an error."""
        with self.assertRaises(TypeError):
            self.manager.get_all('category', 'foo')
